@@ -15,8 +15,6 @@
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA               *
 **************************************************************************/
-#include <ctime>
-
 #include <common/regex.hpp>
 
 #include <cupt/config.hpp>
@@ -122,6 +120,11 @@ CacheImpl::getSatisfyingVersions(const Relation& relation) const
 		{
 			if (relation.isSatisfiedBy((*it)->versionString))
 			{
+				if ((*it)->isInstalled() &&
+						systemState->getInstalledInfo((*it)->packageName)->isBroken())
+				{
+					continue;
+				}
 				result.push_back(*it);
 			}
 		}
@@ -145,6 +148,11 @@ CacheImpl::getSatisfyingVersions(const Relation& relation) const
 				auto versions = reverseProvidePackage->getVersions();
 				FORIT(versionIt, versions)
 				{
+					if ((*versionIt)->isInstalled() &&
+							systemState->getInstalledInfo((*versionIt)->packageName)->isBroken())
+					{
+						continue;
+					}
 					const vector< string >& realProvides = (*versionIt)->provides;
 					FORIT(realProvidesIt, realProvides)
 					{
@@ -330,10 +338,12 @@ void CacheImpl::processIndexEntry(const IndexEntry& indexEntry)
 
 	try
 	{
-		auto releaseInfo = getReleaseInfo(
-				cachefiles::getPathOfReleaseList(*config, indexEntry));
+		auto releaseFilePath = cachefiles::getPathOfReleaseList(*config, indexEntry);
+		auto releaseInfo = cachefiles::getReleaseInfo(*config, releaseFilePath);
 		releaseInfo->component = indexEntry.component;
 		releaseInfo->baseUri = indexEntry.uri;
+		releaseInfo->verified = cachefiles::verifySignature(*config, releaseFilePath);
+
 		if (indexEntry.category == IndexEntry::Binary)
 		{
 			binaryReleaseData.push_back(releaseInfo);
@@ -371,125 +381,7 @@ void CacheImpl::processIndexEntry(const IndexEntry& indexEntry)
 	}
 }
 
-shared_ptr< ReleaseInfo > CacheImpl::getReleaseInfo(const string& path) const
-{
-	shared_ptr< ReleaseInfo > result(new ReleaseInfo);
-	result->notAutomatic = false; // default
 
-	string openError;
-	File file(path, "r", openError);
-	if (!openError.empty())
-	{
-		fatal("unable to open release file '%s': EEE", path.c_str());
-	}
-
-	size_t lineNumber = 1;
-	static sregex fieldRegex = sregex::compile("^((?:\\w|-)+?): (.*)"); // $ implied in regex
-	smatch matches;
-	try
-	{
-		string line;
-		while (! file.getLine(line).eof())
-		{
-			if (!regex_match(line, matches, fieldRegex))
-			{
-				break;
-			}
-			string fieldName = matches[1];
-			string fieldValue = matches[2];
-
-			if (fieldName == "Origin")
-			{
-				result->vendor = fieldValue;
-			}
-			else if (fieldName == "Label")
-			{
-				result->label = fieldValue;
-			}
-			else if (fieldName == "Suite")
-			{
-				result->archive = fieldValue;
-			}
-			else if (fieldName == "Codename")
-			{
-				result->codename = fieldValue;
-			}
-			else if (fieldName == "Date")
-			{
-				result->date = fieldValue;
-			}
-			else if (fieldName == "Valid-Until")
-			{
-				result->validUntilDate = fieldValue;
-			}
-			else if (fieldName == "NotAutomatic")
-			{
-				result->notAutomatic = true;
-			}
-			else if (fieldName == "Architectures")
-			{
-				result->architectures = split(' ', fieldValue);
-			}
-			else if (fieldName == "Description")
-			{
-				result->description = fieldValue;
-				smatch descriptionMatch;
-				if (regex_search(fieldValue, descriptionMatch, sregex::compile("[0-9][0-9a-z._-]*")))
-				{
-					result->version = descriptionMatch[0];
-				}
-			}
-		}
-		++lineNumber;
-	}
-	catch (Exception&)
-	{
-		fatal("error parsing release file '%s', line %u", path.c_str(), lineNumber);
-	}
-
-	if (result->vendor.empty())
-	{
-		warn("no vendor specified in release file '%s'", path.c_str());
-	}
-	if (result->archive.empty())
-	{
-		warn("no archive specified in release file '%s'", path.c_str());
-	}
-
-	{ // checking Valid-Until
-		if (!result->validUntilDate.empty())
-		{
-			struct tm validUntilTm;
-			memset(&validUntilTm, 0, sizeof(validUntilTm));
-			struct tm currentTm;
-
-			auto oldTimeSpec = setlocale(LC_TIME, "C");
-			auto parseResult = strptime(result->validUntilDate.c_str(), "%a, %d %b %Y %T UTC", &validUntilTm);
-			setlocale(LC_TIME, oldTimeSpec);
-			if (parseResult) // success
-			{
-				time_t localTime = time(NULL);
-				gmtime_r(&localTime, &currentTm);
-				// sanely, we should use timegm() here, but it's not portable,
-				// so we use mktime() which is enough for comparing two UTC tm's
-				if (mktime(&currentTm) > mktime(&validUntilTm))
-				{
-					fatal("release file '%s' has expired (expiry time '%s'), discarding it",
-							path.c_str(), result->validUntilDate.c_str());
-				}
-			}
-			else
-			{
-				warn("unable to parse expiry time '%s' in release file '%s'",
-						result->validUntilDate.c_str(), path.c_str());
-			}
-		}
-	}
-
-	result->verified = Cache::verifySignature(config, path);
-
-	return result;
-}
 
 void CacheImpl::processIndexFile(const string& path, IndexEntry::Type category,
 		shared_ptr< const ReleaseInfo > releaseInfo)
